@@ -259,7 +259,7 @@ gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str)
 	default:
 		if (GSF_TIMESTAMP_TYPE == t) {
 			GsfTimestamp ts;
-			if (gsf_timestamp_parse (str, &ts)) {
+			if (gsf_timestamp_from_string (str, &ts)) {
 				gsf_value_set_timestamp (res, &ts);
 				break;
 			}
@@ -419,6 +419,102 @@ gsf_xmlDocFormatDump (GsfOutput *output, xmlDocPtr cur, char const *encoding,
 	return xmlSaveFormatFileTo (buf, cur, encoding, format);
 }
 
+/**************************************************************************/
+/* We parse and do some limited validation of the XML file, if this passes,
+ * then we return TRUE
+ */
+typedef struct {
+	GsfXMLProbeFunc	func;
+	gboolean success;
+} GsfXMLProbeState;
+
+static void
+gsf_xml_probe_error (GsfXMLProbeState *state, char const *msg, ...)
+{
+	(void)msg;
+	state->func = NULL;
+	state->success = FALSE;
+}
+static void
+gsf_xml_probe_element (GsfXMLProbeState *state,
+		       const xmlChar *name,
+		       const xmlChar *prefix,
+		       const xmlChar *URI,
+		       int nb_namespaces,
+		       const xmlChar **namespaces,
+		       int nb_attributes,
+		       int nb_defaulted,
+		       const xmlChar **attributes)
+{
+	state->success = (state->func) (name, prefix, URI, nb_namespaces, namespaces,
+					nb_attributes, nb_defaulted, attributes);
+	state->func = NULL;
+}
+
+gboolean
+gsf_xml_probe (GsfInput *input, GsfXMLProbeFunc func)
+{
+	static xmlSAXHandler gsf_xml_prober = {
+		NULL, /* internalSubset */
+		NULL, /* isStandalone */
+		NULL, /* hasInternalSubset */
+		NULL, /* hasExternalSubset */
+		NULL, /* resolveEntity */
+		NULL, /* getEntity */
+		NULL, /* entityDecl */
+		NULL, /* notationDecl */
+		NULL, /* attributeDecl */
+		NULL, /* elementDecl */
+		NULL, /* unparsedEntityDecl */
+		NULL, /* setDocumentLocator */
+		NULL, /* startDocument */
+		NULL, /* endDocument */
+		NULL, /* startElement */
+		NULL, /* endElement */
+		NULL, /* reference */
+		NULL, /* characters */
+		NULL, /* ignorableWhitespace */
+		NULL, /* processingInstruction */
+		NULL, /* comment */
+		NULL, /* xmlParserWarning */
+		(errorSAXFunc) &gsf_xml_probe_error, /* error */
+		(errorSAXFunc) &gsf_xml_probe_error, /* fatalError */
+		NULL, /* getParameterEntity */
+		NULL, /* cdataBlock; */
+		NULL, /* externalSubset; */
+		XML_SAX2_MAGIC,
+		NULL,
+		(startElementNsSAX2Func) &gsf_xml_probe_element, /* startElementNs */
+		NULL, /* endElementNs */
+		NULL  /* xmlStructuredErrorFunc */
+	};
+	GsfXMLProbeState probe_state = { func, FALSE };
+	xmlParserCtxt *parse_context;
+	char const *buf;
+
+	if (gsf_input_seek (input, 0, G_SEEK_SET))
+		return FALSE;
+
+	g_object_ref (input);
+	input = gsf_input_uncompress (input);
+	gsf_input_seek (input, 0, G_SEEK_SET);
+
+	buf = gsf_input_read (input, 4, NULL);
+	if (NULL != buf ) {
+		parse_context = xmlCreatePushParserCtxt (&gsf_xml_prober, &probe_state,
+			(char *)buf, 4, gsf_input_name (input));
+		if (NULL != parse_context) {
+			while (NULL != probe_state.func &&
+			       NULL != (buf = gsf_input_read (input, 1, NULL)))
+				xmlParseChunk (parse_context, (char *)buf, 1, 0);
+		}
+		xmlFreeParserCtxt (parse_context);
+	}
+	g_object_unref (input);
+
+	return probe_state.success;
+}
+
 /***************************************************************************/
 
 typedef struct {
@@ -519,24 +615,23 @@ lookup_child (GsfXMLInInternal *state, int default_ns_id,
 
 	for (ptr = groups ; ptr != NULL ; ptr = ptr->next) {
 		group = ptr->data;
-		/* does the namespace match */
-		if (group->ns_id >= 0 && group->ns_id != default_ns_id) {
-
-			if ((int)state->ns_by_id->len <= group->ns_id)
-				continue;
-
-			inst = g_ptr_array_index (state->ns_by_id, group->ns_id);
-			if (inst == NULL || 0 != strncmp (name, inst->tag, inst->taglen))
-				continue;
+		/* Is the node explicitly namespaced ? */
+		if (group->ns_id >= 0 &&
+		    group->ns_id < (int)state->ns_by_id->len &&
+		    NULL != (inst = g_ptr_array_index (state->ns_by_id, group->ns_id)) &&
+		    0 == strncmp (name, inst->tag, inst->taglen))
 			tmp = name + inst->taglen;
-		} else {
+		else if (group->ns_id < 0 ||			/* target is unqualified */
+			 group->ns_id == default_ns_id) {	/* target is in default ns */
 #if 0
 			g_return_val_if_fail ((int)state->ns_by_id->len > group->ns_id, FALSE);
 			inst = g_ptr_array_index (state->ns_by_id, group->ns_id);
 			g_warning ("accepted ns = '%s' looking for '%s'", inst->tag, name);
 #endif
 			tmp = name;
-		}
+		} else
+			continue;
+
 		for (elem = group->elem ; elem != NULL ; elem = elem->next) {
 			node = elem->data;
 			if (node->name == NULL || !strcmp (tmp, node->name)) {
@@ -734,6 +829,9 @@ gsf_xml_in_end_element (GsfXMLInInternal *state,
 static void
 gsf_xml_in_characters (GsfXMLInInternal *state, xmlChar const *chars, int len)
 {
+	if (!state->initialized)
+		return;
+
 	if (state->pub.node->has_content != GSF_XML_NO_CONTENT)
 		g_string_append_len (state->pub.content, chars, len);
 }
@@ -786,8 +884,14 @@ gsf_xml_in_end_document (GsfXMLInInternal *state)
 		g_hash_table_destroy (state->ns_prefixes);
 		state->ns_prefixes = NULL;
 
-		g_return_if_fail (state->pub.node == &state->pub.doc->root_node->pub);
-		g_return_if_fail (state->unknown_depth == 0);
+		state->initialized = FALSE;
+
+		if (state->pub.node != &state->pub.doc->root_node->pub) {
+			g_warning ("Document likely damaged.");
+		}
+		if (state->unknown_depth > 0) {
+			g_warning ("Document likely damaged.");
+		}
 	}
 }
 
@@ -1538,7 +1642,8 @@ gsf_xml_out_add_cstr (GsfXMLOut *xout, char const *id,
 				gsf_output_write (xout->output, cur-start, start);
 			start = ++cur;
 			gsf_output_write (xout->output, 6, "&quot;");
-		} else if (*cur < 0x20 && id != NULL) {
+		} else if ((*cur == '\n' || *cur == '\r' || *cur == '\t') &&
+			   id != NULL) {
 			guint8 buf[8];
 			sprintf (buf, "&#%d;", *cur);
 
@@ -1547,8 +1652,8 @@ gsf_xml_out_add_cstr (GsfXMLOut *xout, char const *id,
 			start = ++cur;
 
 			gsf_output_write (xout->output, strlen (buf), buf);
-		} else if (((*cur >= 0x20) && (*cur != 0x7f)) ||
-			   (*cur == '\n') || (*cur == '\r') || (*cur == '\t')) {
+		} else if ((*cur >= 0x20 && *cur != 0x7f) ||
+			   (*cur == '\n' || *cur == '\r' || *cur == '\t')) {
 			cur++;
 		} else {
 			/*
